#!/bin/bash
# TODO: Add mutex (http://wiki.bash-hackers.org/howto/mutex)

readonly EMAIL_SUBJECT_PREFIX="fqe alert"

################################################################################
# Executes the fqe jar for all the supported algorithms and all the LUBM queries.
# Params:
#	$1 maximum memory allocation pool for a Java Virtual Machine (JVM)'s
#   $2 timeout in seconds
#   $3 fqe properties file
#	$4 rdfvs properties file
#	$5 (alert) email to address
#	$6 queries to execute range from
#   $7 queries to execute range to
################################################################################
xmx=$1
timeout_in_seconds=$2
prop_file=$3
rdfvs_prop_file=$4
to_address=$5
queries_to_execute_range_from=$6
queries_to_execute_range_to=$7

echo_parameters() {
	echo "Maximum memory allocation pool for a Java Virtual Machine (JVM)'s: ${xmx}"
	echo "Query execution task timeout: ${timeout_in_seconds} seconds"
	echo "Fragmented query executor properties file: ${prop_file}"
	echo "RDFVS properties file: ${rdfvs_prop_file}"
	echo "Email to address for alert emails: ${to_address}"
	echo "Queries to execute range from: ${queries_to_execute_range_from}"
	echo "Queries to execute range to: ${queries_to_execute_range_to}"
}

################################################################################
# Executes the fqe jar for all the supported algorithms and all the LUBM queries.
# Uses:
#	$xmx
#	$timeout_in_seconds
#	$prop_file
#	$rdfvs_prop_file
#	$to_address
#	$queries_to_execute_range_from
#	$queries_to_execute_range_to
################################################################################
execute_all_with_timeout() {
	send_mail "${EMAIL_SUBJECT_PREFIX}" "Starting fqe executor"
	let timed_out_counter=0
	let executed_queries_counter=0
	for algorithm in "EXHAUSTIVE" "REFORMULATION" "SINGLE_ATOM" "GREEDY" "GREEDY_WITH_CHECK" "SHARP_GREEDY" "SHARP_GREEDY_WITH_CHECK"
	do
		echo "Executing queries using the ${algorithm} fragmentation algorithm."
	    for (( query=$queries_to_execute_range_from; query <= $queries_to_execute_range_to; query++ ))
	    do
	        if [ $query -lt 10 ]; then
	            queryName="0$query"
	        else
	        	queryName=$query
	        fi
	        echo "Executing query ${queryName} using the ${algorithm} fragmentation algorithm."
	        execute_string=$(fqe_execute_string $algorithm $queryName)
	        echo $execute_string
	        execute_with_timeout "${execute_string}"
			if [ $timed_out = 1 ]; then
				timed_out_counter=$((timed_out_counter+1))
				send_mail "FQE alert" "Query ${queryName} execution using the ${algorithm} fragmentation algorithm timed out."
				echo "Query execution task timed out. Restarting database engine."
				restart
				sleep 5m
			else
				executed_queries_counter=$((executed_queries_counter+1))
			fi
			
    	done
	done
	
	send_mail "FQE alert" "fqe executor ended with ${executed_queries_counter} queries correctly executed and ${timed_out_counter} timed out query executions."
	echo "Timed out query executions: ${timed_out_counter}"
}

################################################################################
# Executes the given command with timeout.
# Params:
#   $1 command to execute
# Uses:
#	$timeout_in_seconds
# Returns 1 if timed out 0 otherwise
################################################################################
execute_with_timeout() {
    # start the command in a subshell to avoid problem with pipes
    # (spawn accepts one command)
    command="/bin/sh -c \"$1\""

    expect -c "set echo \"-noecho\"; set timeout $timeout_in_seconds; spawn -noecho $command; expect timeout { exit 1 } eof { exit 0 }"

    timed_out=$?
}

################################################################################
# Build the execution string for fqe jar.
# Params:
#	$1 query fragmentation algorithm
#	$2 query name
# Uses:
#	$xmx
#	$prop_file
#	$rdfvs_prop_file
# Returns execution command string for fqe with the given parameters.
################################################################################
fqe_execute_string() {
	algorithm=$1
	query=$2

	execute_string="java -Xmx${xmx} -jar fqe.jar -pf ${prop_file} -rdfvs_pf ${rdfvs_prop_file} -a '${algorithm}' -q 'Q${query}'"
    echo $execute_string
}

################################################################################
# Sends email (with no attachment) to a single recipient.
# Params:
#	$1 email subject
#	$2 email body
################################################################################
send_mail() {
	subject=$1
	body=$2
	mail -s "${subject}" $to_address <<< "${body}"
}